@@ -1,7 +1,7 @@
 /*
-*   $Id: args.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: args.h,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1999-2001, Darren Hiebert
+*   Copyright (c) 1999-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -16,12 +16,16 @@
 */
 #include "general.h"	/* must always come first */
 
+#include <stdio.h>
+
 /*
 *   DATA DECLARATIONS
 */
 
+typedef enum { ARG_NONE, ARG_STRING, ARG_ARGV, ARG_FILE } argType;
+
 typedef struct sArgs {
-    enum { ARG_NONE, ARG_STRING, ARG_ARGV, ARG_FILE } type;
+    argType type;
     union {
 	struct sStringArgs {
 	    const char* string;
