@@ -1,5 +1,5 @@
 /*
-*   $Id: eiffel.c,v 1.12 2002/02/23 05:32:20 darren Exp $
+*   $Id: eiffel.c,v 1.14 2002/03/01 03:47:29 darren Exp $
 *
 *   Copyright (c) 1998-2002, Darren Hiebert
 *
@@ -263,7 +263,7 @@ static int fileGetc (void)
 	}
     }
     if (Debug > 0  &&  c != EOF)
-	putc (c, stderr);
+	putc (c, errout);
     return c;
 }
 
@@ -1046,7 +1046,8 @@ static void parseFeatureClauses (tokenInfo *const token)
     {
 	if (isKeyword (token, KEYWORD_feature))
 	    parseExport (token);
-	if (! isKeyword (token, KEYWORD_invariant) &&
+	if (! isKeyword (token, KEYWORD_feature) &&
+	    ! isKeyword (token, KEYWORD_invariant) &&
 	    ! isKeyword (token, KEYWORD_indexing))
 	{
 	    if (! parseFeature (token))
@@ -1278,15 +1279,15 @@ extern main (int argc, char** argv)
 		case 's':  SelfReferences  = 1; break;
 		case 'd':  Debug           = 1; break;
 		default:
-		    fprintf (stderr, "%s: unknown option: %c\n", argv [0], arg [1]);
-		    fprintf (stderr, Usage, argv [0]);
+		    fprintf (errout, "%s: unknown option: %c\n", argv [0], arg [1]);
+		    fprintf (errout, Usage, argv [0]);
 		    exit (1);
 		    break;
 	    }
 	}
 	else if (File != NULL)
 	{
-	    fprintf (stderr, Usage, argv [0]);
+	    fprintf (errout, Usage, argv [0]);
 	    exit (1);
 	}
 	else
@@ -1304,7 +1305,7 @@ extern main (int argc, char** argv)
 	PrintReferences = 1;
     if (File == NULL)
     {
-	fprintf (stderr, Usage, argv [0]);
+	fprintf (errout, Usage, argv [0]);
 	exit (1);
     }
 
