@@ -1,5 +1,5 @@
 /*
-*   $Id: fortran.c,v 1.3 2002/02/17 06:04:32 darren Exp $
+*   $Id: fortran.c,v 1.7 2002/10/16 21:47:37 darren Exp $
 *
 *   Copyright (c) 1998-2002, Darren Hiebert
 *
@@ -797,11 +797,11 @@ static void readToken (tokenInfo *const token)
     vStringClear (token->string);
 
 getNextChar:
+    c = getChar ();
+
     token->lineNumber	= getSourceLineNumber ();
     token->filePosition	= getInputFilePosition ();
 
-    c = getChar ();
-
     switch (c)
     {
 	case EOF:  longjmp (Exception, (int) ExceptionEOF);	break;
@@ -983,7 +983,9 @@ static void parseTypeSpec (tokenInfo *const token)
 
 	case KEYWORD_double:
 	    readToken (token);
-	    if (! isKeyword (token, KEYWORD_precision))
+	    if (isKeyword (token, KEYWORD_precision))
+		readToken (token);
+	    else
 		skipToToken (token, TOKEN_STATEMENT_END);
 	    break;
 
@@ -1435,7 +1437,8 @@ static void parseBlockData (tokenInfo *const token)
     if (isKeyword (token, KEYWORD_data))
     {
 	readToken (token);
-	makeFortranTag (token, TAG_BLOCK_DATA);
+	if (isType (token, TOKEN_IDENTIFIER))
+	    makeFortranTag (token, TAG_BLOCK_DATA);
     }
     ancestorPush (token);
     skipToNextStatement (token);
@@ -1464,15 +1467,21 @@ static void parseInternalSubprogramPart (tokenInfo *const token)
     {
 	switch (token->keyword)
 	{
-	    case KEYWORD_function:   parseFunctionSubprogram (token);   break;
-	    case KEYWORD_subroutine: parseSubroutineSubprogram (token); break;
-	    case KEYWORD_recursive:  readToken (token); break;
+	    case KEYWORD_function:
+		parseFunctionSubprogram (token);
+		break;
+
+	    case KEYWORD_subroutine:
+		parseSubroutineSubprogram (token);
+		break;
 
 	    default:
 		if (isTypeSpec (token))
 		    parseTypeSpec (token);
-		else
+		else if (isKeyword (token, KEYWORD_end))
 		    done = TRUE;
+		else
+		    readToken (token);
 		break;
 	}
     } while (! done);
