@@ -1,7 +1,7 @@
 /*
-*   $Id: keyword.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: keyword.c,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -18,8 +18,8 @@
 
 #include "debug.h"
 #include "keyword.h"
-#include "main.h"
 #include "options.h"
+#include "routines.h"
 
 /*
 *   MACROS
@@ -129,9 +129,7 @@ extern void addKeyword (const char *const string, langType language, int value)
     if (entry == NULL)
     {
 	hashEntry **const table = getHashTable ();
-	hashEntry *new = newEntry (string, language, value);
-
-	table [hashedValue] = new;
+	table [hashedValue] = newEntry (string, language, value);
     }
     else
     {
@@ -149,10 +147,8 @@ extern void addKeyword (const char *const string, langType language, int value)
 	}
 	if (entry == NULL)
 	{
-	    hashEntry *new = newEntry (string, language, value);
-
 	    Assert (prev != NULL);
-	    prev->next = new;
+	    prev->next = newEntry (string, language, value);
 	}
     }
 }
