@@ -1,7 +1,7 @@
 /*
-*   $Id: lisp.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: lisp.c,v 1.2 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 2000-2001, Darren Hiebert
+*   Copyright (c) 2000-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -89,7 +89,7 @@ static void findLispTags (void)
 	{
 	    if (L_isdef (p))
 	    {
-		while (!isspace ((int) *p))
+		while (*p != '\0' && !isspace ((int) *p))
 		    p++;
 		while (isspace ((int) *p))
 		    p++;
@@ -100,7 +100,7 @@ static void findLispTags (void)
 		/* Check for (foo::defmumble name-defined ... */
 		do
 		    p++;
-		while (!isspace ((int) *p)
+		while (*p != '\0' && !isspace ((int) *p)
 			&& *p != ':' && *p != '(' && *p != ')');
 		if (*p == ':')
 		{
@@ -110,7 +110,7 @@ static void findLispTags (void)
 
 		    if (L_isdef (p - 1))
 		    {
-			while (!isspace ((int) *p))
+			while (*p != '\0' && !isspace ((int) *p))
 			    p++;
 			while (isspace (*p))
 			    p++;
