@@ -1,5 +1,5 @@
 /*
-*   $Id: options.c,v 1.8 2002/02/25 04:45:45 darren Exp $
+*   $Id: options.c,v 1.12 2002/09/30 04:30:47 darren Exp $
 *
 *   Copyright (c) 1996-2002, Darren Hiebert
 *
@@ -115,7 +115,8 @@ optionValues Option = {
 	FALSE,		/* -fields=K */
 	FALSE,		/* -fields=l */
 	FALSE,		/* -fields=n */
-	TRUE		/* -fields=s */
+	TRUE,		/* -fields=s */
+	FALSE		/* -fields=S */
     },
     NULL,		/* -I */
     FALSE,		/* -a */
@@ -127,7 +128,7 @@ optionValues Option = {
     EX_MIX,		/* -n, --excmd */
 #endif
     FALSE,		/* -R */
-    TRUE,		/* -u, --sort */
+    SO_SORTED,		/* -u, --sort */
     FALSE,		/* -V */
     FALSE,		/* -x */
     NULL,		/* -L */
@@ -204,7 +205,7 @@ static optionDescription LongOptionDescription [] = {
  {1,"  --extra=[+|-]flags"},
  {1,"      Include extra tag entries for selected information (flags: \"fq\")."},
  {1,"  --fields=[+|-]flags"},
- {1,"      Include selected extension fields (flags: \"afmikKlnsz\") [fks]."},
+ {1,"      Include selected extension fields (flags: \"afmikKlnsSz\") [fks]."},
  {1,"  --file-scope=[yes|no]"},
  {1,"       Should tags scoped only for a single file (e.g. \"static\" tags"},
  {1,"       be included in the output [yes]?"},
@@ -252,8 +253,8 @@ static optionDescription LongOptionDescription [] = {
  {1,"  --regex-<LANG>=/line_pattern/name_pattern/[flags]"},
  {1,"       Define regular expression for locating tags in specific language."},
 #endif
- {0,"  --sort=[yes|no]"},
- {0,"       Should tags be sorted [yes]?"},
+ {0,"  --sort=[yes|no|foldcase]"},
+ {0,"       Should tags be sorted (optionally ignoring case) [yes]?."},
  {0,"  --tag-relative=[yes|no]"},
  {0,"       Should paths be relative to location of tag file [no; yes when -e]?"},
  {1,"  --totals=[yes|no]"},
@@ -410,25 +411,12 @@ extern void checkOptions (void)
 	if (Option.tagFileName != NULL)
 	    error (WARNING, "%s ignores output tag file name", notice);
     }
-#ifdef UPDATE_ENABLED
-    if (Option.update)
-    {
-	notice = "update option is not compatible with";
-	if (Option.etags)
-	    error (FATAL, "%s emacs-style tags", notice);
-	if (Option.filter)
-	    error (FATAL, "%s filter option", notice);
-	if (isDestinationStdout ())
-	    error (FATAL, "%s tags to stdout", notice);
-	Option.append = TRUE;
-    }
-#endif
 }
 
 static void setEtagsMode (void)
 {
     Option.etags = TRUE;
-    Option.sorted = FALSE;
+    Option.sorted = SO_UNSORTED;
     Option.lineDirectives = FALSE;
     Option.tagRelative = TRUE;
 }
@@ -684,6 +672,24 @@ extern const char *fileExtension (const char *const fileName)
     return extension;
 }
 
+static boolean isFalse (const char *parameter)
+{
+    return (boolean) (
+	strcasecmp (parameter, "0"  ) == 0  ||
+	strcasecmp (parameter, "n"  ) == 0  ||
+	strcasecmp (parameter, "no" ) == 0  ||
+	strcasecmp (parameter, "off") == 0);
+}
+
+static boolean isTrue (const char *parameter)
+{
+    return (boolean) (
+	strcasecmp (parameter, "1"  ) == 0  ||
+	strcasecmp (parameter, "y"  ) == 0  ||
+	strcasecmp (parameter, "yes") == 0  ||
+	strcasecmp (parameter, "on" ) == 0);
+}
+
 /*  Determines whether the specified file name is considered to be a header
  *  file for the purposes of determining whether enclosed tags are global or
  *  static.
@@ -902,6 +908,21 @@ static void processOptionFile (const char *const option,
 	error (FATAL | PERROR, "cannot open option file \"%s\"", parameter);
 }
 
+static void processSortType (const char *const option,
+			     const char *const parameter)
+{
+    if (isFalse (parameter))
+	Option.sorted = SO_UNSORTED;
+    else if (isTrue (parameter))
+	Option.sorted = SO_SORTED;
+    else if (strcasecmp (parameter, "f") == 0 ||
+	    strcasecmp (parameter, "fold") == 0 ||
+	    strcasecmp (parameter, "foldcase") == 0)
+	Option.sorted = SO_FOLDSORTED;
+    else
+	error (FATAL, "Invalid value for \"%s\" option", option);
+}
+
 static void installHeaderListDefaults (void)
 {
     Option.headerExt = stringListNewFromArgv (HeaderExtensions);
@@ -1131,10 +1152,15 @@ static void processFormatOption (const char *const option,
 static void processEtagsInclude (const char *const __unused__ option,
 				 const char *const parameter)
 {
-    vString *const file = vStringNewInit (parameter);
-    if (Option.etagsInclude == NULL)
-	Option.etagsInclude = stringListNew ();
-    stringListAdd (Option.etagsInclude, file);
+    if (! Option.etags)
+	error (FATAL, "Etags must be enabled to use \"%s\" option", option);
+    else
+    {
+	vString *const file = vStringNewInit (parameter);
+	if (Option.etagsInclude == NULL)
+	    Option.etagsInclude = stringListNew ();
+	stringListAdd (Option.etagsInclude, file);
+    }
 }
 
 static void processFilterTerminatorOption (const char *const __unused__ option,
@@ -1178,6 +1204,7 @@ static void processExtensionFieldsOption (const char *const option,
 	case 'l': field->language	= mode;	break;
 	case 'n': field->lineNumber	= mode;	break;
 	case 's': field->scope		= mode;	break;
+	case 'S': field->signature	= mode;	break;
 	case 'z': field->kindKey	= mode;	break;
 
 	default: error(WARNING, "Unsupported parameter '%c' for \"%s\" option",
@@ -1238,6 +1265,7 @@ static parametricOption ParametricOptions [] = {
     { "langdef",		processLanguageDefineOption,	FALSE	},
     { "langmap",		processLanguageMapOption,	FALSE	},
     { "options",		processOptionFile,		FALSE	},
+    { "sort",			processSortType,		TRUE	},
 };
 
 static booleanOption BooleanOptions [] = {
@@ -1252,7 +1280,6 @@ static booleanOption BooleanOptions [] = {
 #ifdef RECURSE_SUPPORTED
     { "recurse",	&Option.recurse,		FALSE	},
 #endif
-    { "sort",		&Option.sorted,			TRUE	},
     { "tag-relative",	&Option.tagRelative,		TRUE	},
     { "totals",		&Option.printTotals,		TRUE	},
     { "verbose",	&Option.verbose,		FALSE	},
@@ -1296,13 +1323,9 @@ static boolean getBooleanOption (const char *const option,
 
     if (parameter [0] == '\0')
 	selection = TRUE;
-    else if (strcmp (parameter, "0"  ) == 0  ||
-	     strcmp (parameter, "no" ) == 0  ||
-	     strcmp (parameter, "off") == 0)
+    else if (isFalse (parameter))
 	selection = FALSE;
-    else if (strcmp (parameter, "1"  ) == 0  ||
-	     strcmp (parameter, "yes") == 0  ||
-	     strcmp (parameter, "on" ) == 0)
+    else if (isTrue (parameter))
 	selection = TRUE;
     else
 	error (FATAL, "Invalid value for \"%s\" option", option);
@@ -1466,7 +1489,7 @@ static void processShortOption (const char *const option,
 	    break;
 	case 'u':
 	    checkOptionOrder (option);
-	    Option.sorted = FALSE;
+	    Option.sorted = SO_UNSORTED;
 	    break;
 	case 'V':
 	    Option.verbose = TRUE;
