@@ -1,5 +1,5 @@
 /*
-*   $Id: parse.c,v 1.5 2002/02/19 05:18:46 darren Exp $
+*   $Id: parse.c,v 1.6 2002/03/09 22:11:09 darren Exp $
 *
 *   Copyright (c) 1996-2002, Darren Hiebert
 *
@@ -175,8 +175,12 @@ extern langType getFileLanguage (const char *const fileName)
 	if (language == LANG_IGNORE)
 	    language = getPatternLanguage (fileName);
 #ifdef SYS_INTERPRETER
-	if (language == LANG_IGNORE  &&  isExecutable (fileName))
-	    language = getInterpreterLanguage (fileName);
+	if (language == LANG_IGNORE)
+	{
+	    fileStatus *status = eStat (fileName);
+	    if (status->isExecutable)
+		language = getInterpreterLanguage (fileName);
+	}
 #endif
     }
     return language;
