@@ -1,5 +1,5 @@
 /*
-*   $Id: readtags.c,v 1.11 2002/02/17 06:04:32 darren Exp $
+*   $Id: readtags.c,v 1.16 2002/09/16 06:22:39 darren Exp $
 *
 *   Copyright (c) 1996-2002, Darren Hiebert
 *
@@ -16,6 +16,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>	/* declare off_t */
 #endif
@@ -42,8 +43,8 @@ struct sTagFile {
     short initialized;
 	/* format of tag file */
     short format;
-	/* is the tag file sorted? */
-    short sorted;
+	/* how is the tag file sorted? */
+    short sortMethod;
 	/* pointer to file structure */
     FILE* fp;
 	/* file position of first character of `line' */
@@ -97,6 +98,33 @@ const char *const PseudoTagPrefix = "!_";
 *   FUNCTION DEFINITIONS
 */
 
+/*
+ * Compare two strings, ignoring case.
+ * Return 0 for match, < 0 for smaller, > 0 for bigger
+ * Make sure case is folded to uppercase in comparison (like for 'sort -f')
+ * This makes a difference when one of the chars lies between upper and lower
+ * ie. one of the chars [ \ ] ^ _ ` for ascii. (The '_' in particular !)
+ */
+static int struppercmp (const char *s1, const char *s2)
+{
+    int result;
+    do
+    {
+	result = toupper ((int) *s1) - toupper ((int) *s2);
+    } while (result == 0  &&  *s1++ != '\0'  &&  *s2++ != '\0');
+    return result;
+}
+
+static int strnuppercmp (const char *s1, const char *s2, size_t n)
+{
+    int result;
+    do
+    {
+	result = toupper ((int) *s1) - toupper ((int) *s2);
+    } while (result == 0  &&  --n > 0  &&  *s1++ != '\0'  &&  *s2++ != '\0');
+    return result;
+}
+
 static int growString (vstring *s)
 {
     int result = 0;
@@ -348,11 +376,11 @@ static char *duplicate (const char *str)
 static void readPseudoTags (tagFile *const file, tagFileInfo *const info)
 {
     fpos_t startOfLine;
-    size_t prefixLength = strlen (PseudoTagPrefix);
+    const size_t prefixLength = strlen (PseudoTagPrefix);
     if (info != NULL)
     {
-	info->file.format = 1;
-	info->file.sorted = 0;
+	info->file.format     = 1;
+	info->file.sort       = TAG_UNSORTED;
 	info->program.author  = NULL;
 	info->program.name    = NULL;
 	info->program.url     = NULL;
@@ -373,7 +401,7 @@ static void readPseudoTags (tagFile *const file, tagFileInfo *const info)
 	    key = entry.name + prefixLength;
 	    value = entry.file;
 	    if (strcmp (key, "TAG_FILE_SORTED") == 0)
-		file->sorted = atoi (value);
+		file->sortMethod = atoi (value);
 	    else if (strcmp (key, "TAG_FILE_FORMAT") == 0)
 		file->format = atoi (value);
 	    else if (strcmp (key, "TAG_PROGRAM_AUTHOR") == 0)
@@ -387,7 +415,7 @@ static void readPseudoTags (tagFile *const file, tagFileInfo *const info)
 	    if (info != NULL)
 	    {
 		info->file.format     = file->format;
-		info->file.sorted     = file->sorted;
+		info->file.sort       = file->sortMethod;
 		info->program.author  = file->program.author;
 		info->program.name    = file->program.name;
 		info->program.url     = file->program.url;
@@ -398,12 +426,26 @@ static void readPseudoTags (tagFile *const file, tagFileInfo *const info)
     fsetpos (file->fp, &startOfLine);
 }
 
+static void gotoFirstLogicalTag (tagFile *const file)
+{
+    fpos_t startOfLine;
+    const size_t prefixLength = strlen (PseudoTagPrefix);
+    rewind (file->fp);
+    while (1)
+    {
+	fgetpos (file->fp, &startOfLine);
+	if (! readTagLine (file))
+	    break;
+	if (strncmp (file->line.buffer, PseudoTagPrefix, prefixLength) != 0)
+	    break;
+    }
+    fsetpos (file->fp, &startOfLine);
+}
+
 static tagFile *initialize (const char *const filePath, tagFileInfo *const info)
 {
-    tagFile *const result = (tagFile*) malloc (sizeof (tagFile));
-    if (result == NULL)
-	perror ("Cannot open tag file");
-    else
+    tagFile *result = (tagFile*) malloc (sizeof (tagFile));
+    if (result != NULL)
     {
 	memset (result, 0, sizeof (tagFile));
 	growString (&result->line);
@@ -413,13 +455,18 @@ static tagFile *initialize (const char *const filePath, tagFileInfo *const info)
 	    result->fields.max * sizeof (tagExtensionField));
 	result->fp = fopen (filePath, "r");
 	if (result->fp == NULL)
-	    perror ("cannot open tag file");
+	{
+	    free (result);
+	    result = NULL;
+	    info->status.error_number = errno;
+	}
 	else
 	{
 	    fseek (result->fp, 0, SEEK_END);
 	    result->size = ftell (result->fp);
 	    rewind (result->fp);
 	    readPseudoTags (result, info);
+	    info->status.opened = 1;
 	    result->initialized = 1;
 	}
     }
@@ -491,46 +538,22 @@ static int readTagLineSeek (tagFile *const file, const off_t pos)
     return result;
 }
 
-#if !defined (HAVE_STRCASECMP) && !defined (HAVE_STRICMP)
-static int strcasecmp (const char *s1, const char *s2)
-{
-    int result;
-    do
-    {
-	result = toupper ((int) *s1) - toupper ((int) *s2);
-    } while (result == 0  &&  *s1++ != '\0'  &&  *s2++ != '\0');
-    return result;
-}
-#endif
-
-#if !defined (HAVE_STRNCASECMP) && !defined (HAVE_STRNICMP)
-static int strncasecmp (const char *s1, const char *s2, size_t n)
-{
-    int result;
-    do
-    {
-	result = toupper ((int) *s1) - toupper ((int) *s2);
-    } while (result == 0  &&  --n > 0  &&  *s1++ != '\0'  &&  *s2++ != '\0');
-    return result;
-}
-#endif
-
 static int nameComparison (tagFile *const file)
 {
     int result;
     if (file->search.ignorecase)
     {
 	if (file->search.partial)
-	    result = strncasecmp (file->search.name, file->name.buffer,
-			       file->search.nameLength);
+	    result = strnuppercmp (file->search.name, file->name.buffer,
+		    file->search.nameLength);
 	else
-	    result = strcasecmp (file->search.name, file->name.buffer);
+	    result = struppercmp (file->search.name, file->name.buffer);
     }
     else
     {
 	if (file->search.partial)
 	    result = strncmp (file->search.name, file->name.buffer,
-			      file->search.nameLength);
+		    file->search.nameLength);
 	else
 	    result = strcmp (file->search.name, file->name.buffer);
     }
@@ -638,10 +661,21 @@ static tagResult find (tagFile *const file, tagEntry *const entry,
     fseek (file->fp, 0, SEEK_END);
     file->size = ftell (file->fp);
     rewind (file->fp);
-    if (file->sorted && ! file->search.ignorecase)
+    if ((file->sortMethod == TAG_SORTED      && !file->search.ignorecase) ||
+	(file->sortMethod == TAG_FOLDSORTED  &&  file->search.ignorecase))
+    {
+#ifdef DEBUG
+	printf ("<performing binary search>\n");
+#endif
 	result = findBinary (file);
+    }
     else
+    {
+#ifdef DEBUG
+	printf ("<performing sequential search>\n");
+#endif
 	result = findSequential (file);
+    }
 
     if (result != TagSuccess)
 	file->search.pos = file->size;
@@ -657,7 +691,8 @@ static tagResult find (tagFile *const file, tagEntry *const entry,
 static tagResult findNext (tagFile *const file, tagEntry *const entry)
 {
     tagResult result = TagFailure;
-    if (file->sorted && ! file->search.ignorecase)
+    if ((file->sortMethod == TAG_SORTED      && !file->search.ignorecase) ||
+	(file->sortMethod == TAG_FOLDSORTED  &&  file->search.ignorecase))
     {
 	result = tagsNext (file, entry);
 	if (result == TagSuccess  && nameComparison (file) != 0)
@@ -681,17 +716,28 @@ extern tagFile *tagsOpen (const char *filePath, tagFileInfo *info)
     return initialize (filePath, info);
 }
 
-extern tagResult tagsSetSorted (tagFile *file, int sorted)
+extern tagResult tagsSetSortType (tagFile *file, sortType type)
 {
     tagResult result = TagFailure;
     if (file != NULL  &&  file->initialized)
     {
-	file->sorted = sorted;
+	file->sortMethod = type;
 	result = TagSuccess;
     }
     return result;
 }
 
+extern tagResult tagsFirst (tagFile *const file, tagEntry *const entry)
+{
+    tagResult result = TagFailure;
+    if (file != NULL  &&  file->initialized)
+    {
+	gotoFirstLogicalTag (file);
+	result = readNext (file, entry);
+    }
+    return result;
+}
+
 extern tagResult tagsNext (tagFile *const file, tagEntry *const entry)
 {
     tagResult result = TagFailure;
@@ -743,9 +789,10 @@ extern tagResult tagsClose (tagFile *file)
 #ifdef READTAGS_MAIN
 
 static const char *TagFileName = "tags";
+static const char *ProgramName;
 static int extensionFields;
 static int SortOverride;
-static int Sorted;
+static int SortMethod;
 
 static void printTag (const tagEntry *entry)
 {
@@ -781,11 +828,15 @@ static void findTag (const char *const name, const int options)
     tagEntry entry;
     tagFile *const file = tagsOpen (TagFileName, &info);
     if (file == NULL)
-	perror ("Cannot open tag file");
+    {
+	fprintf (stderr, "%s: cannot open tag file: %s: %s\n",
+		ProgramName, strerror (info.status.error_number), name);
+	exit (1);
+    }
     else
     {
 	if (SortOverride)
-	    tagsSetSorted (file, Sorted);
+	    tagsSetSortType (file, SortMethod);
 	if (tagsFind (file, &entry, name, options) == TagSuccess)
 	{
 	    do
@@ -803,7 +854,11 @@ static void listTags (void)
     tagEntry entry;
     tagFile *const file = tagsOpen (TagFileName, &info);
     if (file == NULL)
-	perror ("Cannot open tag file");
+    {
+	fprintf (stderr, "%s: cannot open tag file: %s: %s\n",
+		ProgramName, strerror (info.status.error_number), TagFileName);
+	exit (1);
+    }
     else
     {
 	while (tagsNext (file, &entry) == TagSuccess)
@@ -820,51 +875,83 @@ const char *const Usage =
     "    -i           Perform case-insensitive matching.\n"
     "    -l           List all tags.\n"
     "    -p           Perform partial matching.\n"
-    "    -s[0|1]      Override sort detection of tag file.\n"
+    "    -s[0|1|2]    Override sort detection of tag file.\n"
     "    -t file      Use specified tag file (default: \"tags\").\n"
     "Note that options are acted upon as encountered, so order is significant.\n";
 
 extern int main (int argc, char **argv)
 {
-    int i;
     int options = 0;
+    int actionSupplied = 0;
+    int i;
+    ProgramName = argv [0];
     if (argc == 1)
     {
-	fprintf (stderr, Usage, argv [0]);
+	fprintf (errout, Usage, ProgramName);
 	exit (1);
     }
     for (i = 1  ;  i < argc  ;  ++i)
     {
 	const char *const arg = argv [i];
-	if (arg [0] == '-')
+	if (arg [0] != '-')
+	{
+	    findTag (arg, options);
+	    actionSupplied = 1;
+	}
+	else
 	{
 	    int j;
 	    for (j = 1  ;  arg [j] != '\0'  ;  ++j)
 	    {
-		switch (arg[j])
+		switch (arg [j])
 		{
 		    case 'e': extensionFields = 1;         break;
 		    case 'i': options |= TAG_IGNORECASE;   break;
 		    case 'p': options |= TAG_PARTIALMATCH; break;
-		    case 'l': listTags ();                 break;
-		    case 't': TagFileName = argv [++i];    break;
+		    case 'l': listTags (); actionSupplied = 1; break;
+	    
+		    case 't':
+			if (arg [j+1] != '\0')
+			{
+			    TagFileName = arg + j + 1;
+			    j += strlen (TagFileName);
+			}
+			else if (i + 1 < argc)
+			    TagFileName = argv [++i];
+			else
+			{
+			    fprintf (errout, Usage, ProgramName);
+			    exit (1);
+			}
+			break;
 		    case 's':
 			SortOverride = 1;
+			++j;
 			if (arg [j] == '\0')
-			    Sorted = 1;
+			    SortMethod = 1;
+			else if (strchr ("012", arg[j]) != NULL)
+			    SortMethod = arg[j] - '0';
 			else
-			    Sorted = (arg [++j] != '0');
+			{
+			    fprintf (errout, Usage, ProgramName);
+			    exit (1);
+			}
 			break;
 		    default:
-			fprintf (stderr, "%s: unknown option: %c\n",
-				    argv[0], arg[j]);
+			fprintf (errout, "%s: unknown option: %c\n",
+				    ProgramName, arg[j]);
 			exit (1);
 			break;
 		}
 	    }
 	}
-	else
-	    findTag (arg, options);
+	if (! actionSupplied)
+	{
+	    fprintf (errout,
+		"%s: no action specified: specify tag name(s) or -l option\n",
+		ProgramName);
+	    exit (1);
+	}
     }
     return 0;
 }
