@@ -1,7 +1,7 @@
 /*
-*   $Id: rexx.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: rexx.c,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 2001, Darren Hiebert
+*   Copyright (c) 2001-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -22,8 +22,8 @@
 
 static void installRexxRegex (const langType language)
 {
-    addLanguageRegex (language,
-	"/^([A-Za-z0-9@#$\\.!?_]+)[ \t]*:/\\1/s,subroutine/");
+    addTagRegex (language, "^([A-Za-z0-9@#$\\.!?_]+)[ \t]*:",
+	"\\1", "s,subroutine", NULL);
 }
 
 extern parserDefinition* RexxParser (void)
