@@ -1,7 +1,7 @@
 /*
-*   $Id: sh.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: sh.c,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 2000-2001, Darren Hiebert
+*   Copyright (c) 2000-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -17,9 +17,9 @@
 
 #include <string.h>
 
-#include "main.h"
 #include "parse.h"
 #include "read.h"
+#include "routines.h"
 #include "vstring.h"
 
 /*
