@@ -1,5 +1,5 @@
 /*
- *   $Id: slang.c,v 1.2 2001/12/23 20:31:47 darren Exp $
+ *   $Id: slang.c,v 1.3 2002/10/17 21:11:12 darren Exp $
  *
  *   Copyright (c) 2000-2001, Francesc Rocher
  *
@@ -14,7 +14,7 @@
 /*
  *   INCLUDE FILES
  */
-#include "general.h"
+#include "general.h"	/* must always come first */
 #include "parse.h"
 
 /*
@@ -22,20 +22,20 @@
  */
 static void installSlangRegex (const langType language)
 {
-   addTagRegex (language,
+    addTagRegex (language,
 	"^.*define[ \t]+([A-Z_][A-Z0-9_]*)[^;]*$",
 	"\\1", "f,function", "i");
-   addTagRegex (language,
+    addTagRegex (language,
 	"^[ \t]*implements[ \t]+\\([ \t]*\"([^\"]*)\"[ \t]*\\)[ \t]*;",
 	"\\1", "n,namespace", NULL);
 }
 
 extern parserDefinition* SlangParser (void)
 {
-   static const char *const extensions [] = { "sl", NULL };
-   parserDefinition* const def = parserNew ("SLang");
-   def->extensions = extensions;
-   def->initialize = installSlangRegex;
-   def->regex      = TRUE;
-   return def;
+    static const char *const extensions [] = { "sl", NULL };
+    parserDefinition* const def = parserNew ("SLang");
+    def->extensions = extensions;
+    def->initialize = installSlangRegex;
+    def->regex      = TRUE;
+    return def;
 }
